<?php
add_action('admin_menu', 'buzzblog_add_box_page');

function buzzblog_add_box_page() {
	global $buzzblog_meta_box_page;
	$buzzblog_meta_box_page = array(
	'id' => 'my-meta-box-page',
	'title' =>  esc_html__('Page Options', 'buzzblog'),
	'page' => 'page',
	'context' => 'normal',
	'priority' => 'high',
	'fields' => array(
    	array(
    	   'name' => esc_html__('Title', 'buzzblog'),
    	   'desc' => esc_html__('Input title for the page.', 'buzzblog'),
    	   'id' => 'buzzblog_page_tit',
    	   'type' => 'text',
    	   'std' => ''
    	),
    	array(
    	   'name' => esc_html__('Subtitle', 'buzzblog'),
    	   'desc' => esc_html__('Input subtitle for the page.', 'buzzblog'),
    	   'id' => 'buzzblog_page_sub',
    	   'type' => 'textarea',
    	   'std' => ''
    	),
				array( "name" => esc_html__('Enable / disable title','buzzblog'),
				"desc" => esc_html__('Enable or disable title and subtitle.','buzzblog'),
				"id" => "buzzblog_page_title_enable",
				"type" => "select",
				"std" => esc_html__('enable','buzzblog'),
				"options" => array(esc_html__('enable','buzzblog'), esc_html__('disable','buzzblog')) 
			)
			
	)
);
	add_meta_box($buzzblog_meta_box_page['id'], $buzzblog_meta_box_page['title'], 'buzzblog_show_box_page', $buzzblog_meta_box_page['page'], $buzzblog_meta_box_page['context'], $buzzblog_meta_box_page['priority']);

}


/*-----------------------------------------------------------------------------------*/
/*	Callback function to show fields in meta box
/*-----------------------------------------------------------------------------------*/

function buzzblog_show_box_page() {
	global $buzzblog_meta_box_page, $post;
 	
	echo '<p style="padding:10px 0 0 0;">'.esc_html__('Please fill additional fields for page. ', 'buzzblog').'</p>';
	// Use nonce for verification
	echo '<input type="hidden" name="my_meta_box_nonce" value="', wp_create_nonce(basename(__FILE__)), '" />';
 
	echo '<table class="form-table">';
 
	foreach ($buzzblog_meta_box_page['fields'] as $field) {
		// get current post meta data
		$meta = get_post_meta($post->ID, $field['id'], true);
		switch ($field['type']) {
 
			
			//If Text		
			case 'text':
			
			echo '<tr style="border-top:1px solid #eeeeee;">',
				'<th style="width:25%"><label for="', $field['id'], '"><strong>', $field['name'], '</strong><span style=" display:block; color:#999; margin:5px 0 0 0; line-height: 18px;">'. $field['desc'].'</span></label></th>',
				'<td>';
			echo '<input type="text" name="', $field['id'], '" id="', $field['id'], '" value="', $meta ? $meta : stripslashes(htmlspecialchars(( $field['std']), ENT_QUOTES)), '" size="30" style="width:75%; margin-right: 20px; float:left;" />';
			
			break;
			
			//If textarea		
			case 'textarea':
			
			echo '<tr style="border-top:1px solid #eeeeee;">',
				'<th style="width:25%"><label for="', $field['id'], '"><strong>', $field['name'], '</strong><span style="line-height:18px; display:block; color:#999; margin:5px 0 0 0;">'. $field['desc'].'</span></label></th>',
				'<td>';
			echo '<textarea name="', $field['id'], '" id="', $field['id'], '" value="', $meta ? $meta : stripslashes(htmlspecialchars(( $field['std']), ENT_QUOTES)), '" rows="8" cols="5" style="width:100%; margin-right: 20px; float:left;">', $meta ? $meta : stripslashes(htmlspecialchars(( $field['std']), ENT_QUOTES)), '</textarea>';
			
			break;
			
			//If Select	
			case 'select':
			
				echo '<tr>',
				'<th style="width:25%"><label for="', $field['id'], '"><strong>', $field['name'], '</strong><span style=" display:block; color:#999; margin:5px 0 0 0; line-height: 18px;">'. $field['desc'].'</span></label></th>',
				'<td>';
			
				echo'<select id="' . $field['id'] . '" name="'.$field['id'].'">';
			
				foreach ($field['options'] as $option) {
					
					echo'<option';
					if ($meta == $option ) { 
						echo ' selected="selected"'; 
					}
					echo'>'. $option .'</option>';
				
				} 
				
				echo'</select>';
			
			break; 
			
		}

	}
 
	echo '</table>';
}

 
add_action('save_post', 'buzzblog_save_data_page');


/*-----------------------------------------------------------------------------------*/
/*	Save data when post is edited
/*-----------------------------------------------------------------------------------*/
 
function buzzblog_save_data_page($post_id) {
	global $buzzblog_meta_box_page;
 
	// verify nonce
	if (!isset($_POST['my_meta_box_nonce']) || !wp_verify_nonce($_POST['my_meta_box_nonce'], basename(__FILE__))) {
		return $post_id;
	}
 
	// check autosave
	if (defined('DOING_AUTOSAVE') && DOING_AUTOSAVE) {
		return $post_id;
	}
 
	// check permissions
	if ('page' == $_POST['post_type']) {
		if (!current_user_can('edit_page', $post_id)) {
			return $post_id;
		}
	} elseif (!current_user_can('edit_post', $post_id)) {
		return $post_id;
	}
 
	foreach ($buzzblog_meta_box_page['fields'] as $field) {
		$old = get_post_meta($post_id, $field['id'], true);
		$new = sanitize_text_field($_POST[$field['id']]);
 
		if ($new && $new != $old) {
			update_post_meta($post_id, $field['id'], $new);
		} elseif ('' == $new && $old) {
			delete_post_meta($post_id, $field['id'], $old);
		}
	}
	
}